"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PaginationService = void 0;
const common_1 = require("@nestjs/common");
const pagination_constants_1 = require("./pagination.constants");
const sequelize_typescript_1 = require("sequelize-typescript");
const common_2 = require("./common");
const exceptions_1 = require("./exceptions");
let PaginationService = exports.PaginationService = class PaginationService {
    constructor(options, sequelize) {
        this.options = options;
        this.sequelize = sequelize;
    }
    async findAllPaginate(model, options, optionsSequelize = {}) {
        const modelName = model.name;
        if (!(modelName in this.sequelize.models)) {
            throw new exceptions_1.ModelDoesNotRegisteredException(modelName);
        }
        const mergedOptions = Object.assign({
            path: null,
            details: true,
            offset: 50,
            page: 1,
        }, this.options, options);
        const end = mergedOptions.page * mergedOptions.offset;
        const start = end - mergedOptions.offset;
        const itemCount = mergedOptions.offset;
        const data = await model.findAndCountAll({
            ...optionsSequelize,
            limit: itemCount,
            offset: start,
        });
        const totalItemCount = data.count;
        const totalPages = totalItemCount > 0 ? Math.ceil(totalItemCount / itemCount) : 0;
        let aux = mergedOptions.page + 1;
        const nextPageNumber = aux <= totalPages ? aux : null;
        aux = mergedOptions.page - 1;
        const prevPageNumber = aux >= 1 ? aux : null;
        let url = null;
        let nextPageUrl = null;
        let prevPageUrl = null;
        if (mergedOptions.url) {
            url = new URL(mergedOptions.url);
            if (mergedOptions.path) {
                url.pathname =
                    url.pathname.replace(/\/+$/g, '') +
                        '/' +
                        mergedOptions.path.replace(/^\/+/g, '');
            }
            url.searchParams.set('page', mergedOptions.page.toString());
            url.searchParams.set('offset', mergedOptions.offset.toString());
            if (nextPageNumber) {
                nextPageUrl = new URL(url.toString());
                nextPageUrl.searchParams.set('page', nextPageNumber.toString());
            }
            if (prevPageNumber) {
                prevPageUrl = new URL(url.toString());
                prevPageUrl.searchParams.set('page', prevPageNumber.toString());
            }
        }
        let meta = {
            pageCount: totalPages,
            page: mergedOptions.page,
            nextPage: nextPageNumber,
            prevPage: prevPageNumber,
        };
        if (mergedOptions.details) {
            meta = {
                ...meta,
                offset: mergedOptions.offset,
                totalItems: totalItemCount,
                itemCount: data.rows.length,
            };
            if (url) {
                const firstUrl = new URL(url.toString());
                firstUrl.searchParams.set('page', '1');
                let lastUrl = firstUrl;
                if (totalPages > 0) {
                    lastUrl = new URL(url.toString());
                    lastUrl.searchParams.set('page', totalPages.toString());
                }
                meta = {
                    ...meta,
                    links: {
                        self: url,
                        first: firstUrl,
                        last: lastUrl,
                        next: nextPageUrl,
                        previous: prevPageUrl,
                    },
                };
            }
        }
        return new common_2.PaginatedData(meta, data.rows);
    }
};
exports.PaginationService = PaginationService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(pagination_constants_1.PAGINATION_MODULE_OPTIONS)),
    __metadata("design:paramtypes", [Object, sequelize_typescript_1.Sequelize])
], PaginationService);
